@@ -30,6 +30,9 @@
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
+
+// clazy:excludeall=non-pod-global-static
+
 #ifndef QT_NO_ICON
 #include "xdgiconloader_p.h"
 
@@ -50,10 +53,6 @@
 #include <QXmlStreamReader>
 #include <QFileSystemWatcher>
 
-#ifdef Q_DEAD_CODE_FROM_QT4_MAC
-#include <private/qt_cocoa_helpers_mac_p.h>
-#endif
-
 #include <private/qhexstring_p.h>
 
 //QT_BEGIN_NAMESPACE
@@ -626,11 +625,8 @@ void XdgIconLoaderEngine::paint(QPainter *painter, const QRect &rect,
                              QIcon::Mode mode, QIcon::State state)
 {
     QSize pixmapSize = rect.size();
-#if defined(Q_DEAD_CODE_FROM_QT4_MAC)
-    pixmapSize *= qt_mac_get_scalefactor();
-#endif
     const qreal dpr = painter->device()->devicePixelRatioF();
-    painter->drawPixmap(rect, pixmap(pixmapSize * dpr, mode, state));
+    painter->drawPixmap(rect, pixmap(QSizeF(pixmapSize * dpr).toSize(), mode, state));
 }
 
 /*
@@ -822,7 +818,17 @@ QPixmap ScalableEntry::pixmap(const QSize &size, QIcon::Mode mode, QIcon::State
         svgIcon = QIcon(filename);
 
     // Simply reuse svg icon engine
-    return svgIcon.pixmap(size, mode, state);
+    QPixmap pm = svgIcon.pixmap(size, mode, state);
+    // WARNING: The SVG icon engine gives a too big pixmap with scale factors > 1,
+    // so that the final drawing will not be sharp if the pixmap is not scaled here.
+    // The cause of this old behavior is unknown to me (@tsujan).
+    if (!pm.isNull())
+    {
+        const auto actualSize = svgIcon.actualSize(size, mode, state);
+        if (actualSize != pm.size())
+            pm = pm.scaled(actualSize, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
+    }
+    return pm;
 }
 
 static const QString STYLE = QStringLiteral("\n.ColorScheme-Text, .ColorScheme-NeutralText {color:%1;}\
@@ -922,6 +928,14 @@ QPixmap ScalableFollowsColorEntry::pixmap(const QSize &size, QIcon::Mode mode, Q
         }
     }
 
+    // see ScalableEntry::pixmap() for the reason
+    if (!pm.isNull())
+    {
+        const auto actualSize = svgIcon.actualSize(size, mode, state);
+        if (actualSize != pm.size())
+            pm = pm.scaled(actualSize, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
+    }
+
     return pm;
 }
 
